#***************************************************************************
#* SPDX-FileCopyrightText: 2024 S. MANKOWSKI stephane@mankowski.fr
#* SPDX-FileCopyrightText: 2024 G. DE BURE support@mankowski.fr
#* SPDX-License-Identifier: GPL-3.0-or-later
#***************************************************************************
# works on all platforms
import os

# get the directory containing your running .sikuli
myPath = os.path.dirname(getBundlePath())
if not myPath in sys.path: sys.path.append(myPath)
import shared
try:
    setAutoWaitTimeout(10)
    
    shared.initSimple()
    
    click("Units-1.png")
    sleep(1)
    type(Key.ENTER, KEY_CTRL)
    click("Manual.png")
    paste(Pattern("1305987974969.png").similar(0.82), "unit1")
    paste("5ymbl.png", "u1")
    sleep(1)
    type(Key.ENTER, KEY_CTRL)
    sleep(1)
    type(Key.ENTER, KEY_SHIFT)
    doubleClick(find("1383487091620.png").below().find("unit1u1.png"))
    shared.closeCurrentPage()
    click("AfghanAfghan.png")
    click("Download.png")
    
    wait(2)
    
    click("1383499105796.png")
    
    shared.close()
    pass
except FindFailed:
    shared.generateErrorCapture("unit")
    raise



#***************************************************************************
#* SPDX-FileCopyrightText: 2024 S. MANKOWSKI stephane@mankowski.fr
#* SPDX-FileCopyrightText: 2024 G. DE BURE support@mankowski.fr
#* SPDX-License-Identifier: GPL-3.0-or-later
#***************************************************************************
# works on all platforms
import os

# get the directory containing your running .sikuli
myPath = os.path.dirname(getBundlePath())
if not myPath in sys.path: sys.path.append(myPath)
import shared
try:
    setAutoWaitTimeout(10)
    
    shared.initAllPlugins()
    sleep(5)
    click("1306052486290.png")
    status=find("YStatus.png")
    doubleClick(status)
    wait(1)
    doubleClick(status.nearby(400).find(Pattern("Status-1.png").similar(0.81)))
    click("1306052650498.png")
    click("1306052703527.png")
    sleep(1)
    type(Key.ENTER, KEY_CTRL)
    sleep(1)
    type(Key.ENTER, KEY_SHIFT)
    click("1306053834224.png")
    click("1306053853927.png")
    click("1306053869838.png")
    click("1306053885688.png")
    
    click("1306055505239.png")
    r=find("Operations-1.png").right()
    click(r.find("1305987216474.png"))
    
    click("Update.png")
    status2=status.below().find("YStatus.png")
    doubleClick(status2)
    wait(1)
    doubleClick(status2.nearby(400).find(Pattern("Status-1.png").similar(0.81)))
    click("1306052650498.png")
    click("1306052703527.png")
    click("1305986344895.png")
    sleep(1)
    type(Key.ENTER, KEY_SHIFT)
    
    shared.close()
    pass
except FindFailed:
    shared.generateErrorCapture("search")
    raise



#***************************************************************************
#* SPDX-FileCopyrightText: 2024 S. MANKOWSKI stephane@mankowski.fr
#* SPDX-FileCopyrightText: 2024 G. DE BURE support@mankowski.fr
#* SPDX-License-Identifier: GPL-3.0-or-later
#***************************************************************************
# works on all platforms
import os

# get the directory containing your running .sikuli
myPath = os.path.dirname(getBundlePath())
if not myPath in sys.path: sys.path.append(myPath)
import shared
try:
    setAutoWaitTimeout(10)
    
    shared.initAllPlugins()
    
    click("1446153094651.png")
    click("1446153115180.png")
    click("1446153142941.png")
    sleep(1)
    type(Key.ENTER, KEY_CTRL)
    click("1446155010281.png")
    sleep(2)
    type(Key.ENTER, KEY_CTRL)
    click("1446654721017.png")
    
    wait(10)
    
    shared.undo()
    shared.redo()
    
    shared.close()
    pass
except FindFailed:
    shared.generateErrorCapture("budget")
    raise



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ***************************************************************************
# * SPDX-FileCopyrightText: 2024 S. MANKOWSKI stephane@mankowski.fr
# * SPDX-FileCopyrightText: 2024 G. DE BURE support@mankowski.fr
# * SPDX-License-Identifier: GPL-3.0-or-later
# ***************************************************************************

import argparse
import datetime
import os
import shutil
import subprocess
import sys
import fileinput

__VERSION__ = "1.0.0"

toolPath = os.path.dirname(os.path.realpath(__file__))
localPath = os.path.dirname(toolPath)
tempDir = "/data"


class Releasor(object):
    def __init__(self, args):
        self.versionstotreat = [
            {"v": "5", "ubuntuVersions": ["noble", "oracular"]},
            {"v": "6", "ubuntuVersions": ["oracular"]},
        ]
        self.workdingDir = os.path.join(tempDir, "skrooge-release_" + args.version)
        print("# Working directory :" + self.workdingDir)
        if args.version.endswith(".0") or args.stable:
            self.ppa = "ppa"
            self.ppatotreat = ["beta", self.ppa]
        else:
            self.ppa = "beta"
            self.ppatotreat = [self.ppa]

        if os.path.exists(self.workdingDir):
            self.logfile = open(os.path.join(self.workdingDir, "log.txt"), "w")
        else:
            self.logfile = None

    def prepareWorkingDirectory(self, args):
        print("# Prepare the working directory")
        if os.path.exists(self.workdingDir):
            print("#   Remove " + self.workdingDir)
            shutil.rmtree(self.workdingDir)
        os.mkdir(self.workdingDir)
        os.chdir(self.workdingDir)

        self.logfile = open(os.path.join(self.workdingDir, "log.txt"), "w")
        print("#   DONE")
        return 0

    def makeTarFile(self, args):
        rc = 0
        if args.fromtar:
            print("# Make the tar file from " + args.fromtar)
            dst = os.path.join(self.workdingDir, "skrooge-" + args.version + ".tar.xz")
            shutil.copyfile(args.fromtar, dst)

            print("#   DONE")
        else:
            print("# Make the tar file")
            cmd = ["git", "clone", "https://github.com/KDE/releaseme.git"]
            self.logfile.write("### " + " ".join(cmd) + "\n")
            rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
            if rc == 0:
                print("# Launch tarme.rb")
                cmd = [
                    "releaseme/tarme.rb",
                    "--version",
                    args.version,
                    "--origin",
                    "trunk",
                    "skrooge",
                ]
                self.logfile.write("### " + " ".join(cmd) + "\n")
                rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)

                shutil.rmtree(os.path.join(self.workdingDir, "skrooge-" + args.version))
                shutil.rmtree("releaseme")
                os.remove(os.path.join(self.workdingDir, "release_data"))
                sig_file = os.path.join(
                    self.workdingDir, "skrooge-" + args.version + ".tar.xz.sig"
                )
                if os.path.exists(sig_file):
                    os.remove(sig_file)
            print("#   " + ("DONE" if rc == 0 else "FAILED"))
        return rc

    def updateTarFile(self, args):
        print("# Update the tar file")
        os.chdir(self.workdingDir)
        p = "skrooge-" + args.version
        if os.path.exists(p):
            shutil.rmtree(p)
        tarfile = "skrooge-" + args.version + ".tar.xz"
        print("#   Untar " + tarfile)
        cmd = ["tar", "-xvf", tarfile]
        self.logfile.write("### " + " ".join(cmd) + "\n")
        rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
        if rc == 0:
            print("#   Change the CMakeLists.txt")
            with open(os.path.join(p, "CMakeLists.txt"), "r") as fileCMakeLists:
                CMakeListsContent = ""
                for line in fileCMakeLists:
                    if line.startswith("SET(SKG_VERSION"):
                        print('#     SKG_VERSION changes to "' + args.version + '"')
                        CMakeListsContent += 'SET(SKG_VERSION "' + args.version + '")\n'
                    else:
                        if line.startswith("SET(SKG_BETA"):
                            bb = "BETA" if self.ppa == "beta" else ""
                            print('#     SKG_BETA changes to "' + bb + '"')
                            CMakeListsContent += 'SET(SKG_BETA "' + bb + '")\n'
                        else:
                            if line.startswith("FEATURE_SUMMARY"):
                                CMakeListsContent += line
                                break
                            else:
                                CMakeListsContent += line
            with open(os.path.join(p, "CMakeLists.txt"), "w") as fileCMakeLists:
                fileCMakeLists.write(CMakeListsContent)

            # for line in fileinput.input(['skrooge/org.kde.skrooge.appdata.xml'], inplace=True):
            #    print(line.replace('<releases>', '<releases>\n<release version="'+args.version+'" date="'+datetime.datetime.today().strftime('%Y-%m-%d')+'"/>'), end='')

            print("#   Build the new splash screen")
            buildPath = os.path.join(p, "build")
            os.mkdir(buildPath)
            os.chdir(buildPath)
            cmd = [
                "cmake",
                "..",
                "-DCMAKE_INSTALL_PREFIX=/usr",
                "-DQT_PLUGIN_INSTALL_DIR=/usr/lib/x86_64-linux-gnu/qt6/plugins",
                "-DQT_MAJOR_VERSION=6",
            ]
            self.logfile.write("### " + " ".join(cmd) + "\n")
            rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
            if rc == 0:
                cmd = ["make", "splash"]
                self.logfile.write("### " + " ".join(cmd) + "\n")
                rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
                if rc == 0:
                    print("#   Create the tar file")
                    os.chdir(self.workdingDir)
                    shutil.rmtree(buildPath)
                    os.remove(tarfile)
                    cmd = ["tar", "-cJf", tarfile, p]
                    rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
                    self.logfile.write("### " + " ".join(cmd) + "\n")
        if rc == 0:
            shutil.rmtree(p)
        print("#   " + ("DONE" if rc == 0 else "FAILED"))
        return rc

    def generatePublicationMessage(self, args, filename):
        cmd = ["gpg2", "--armor", "--detach-sig", "-o", filename + ".sig", filename]
        self.logfile.write("### " + " ".join(cmd) + "\n")
        rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
        if rc == 0:
            print(
                "Skrooge "
                + args.version
                + " released\n\nHi,\n\nCould you publish the following files in "
                + ("stable" if args.stable else "unstable")
                + "/skrooge ?\n"
            )
            cmd = ["sha256sum", filename]
            self.logfile.write("### " + " ".join(cmd) + "\n")
            rc = subprocess.call(cmd)
            if rc == 0:
                cmd = ["sha256sum", filename + ".sig"]
                self.logfile.write("### " + " ".join(cmd) + "\n")
                rc = subprocess.call(cmd)
            print("Thank you.\nRegards.\n")

    def buildDebian(self, args):
        print("# Get dsc")
        for v in self.versionstotreat:
            os.chdir(self.workdingDir)
            cmd = ["apt", "source", "skrooge"]
            self.logfile.write("### " + " ".join(cmd) + "\n")
            rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
            if rc == 0:
                previousPackage = [
                    os.path.join(self.workdingDir, o)
                    for o in os.listdir(self.workdingDir)
                    if os.path.isdir(os.path.join(self.workdingDir, o))
                    and o.startswith("skrooge-")
                    and not o.endswith(".orig")
                ][0]
                print("#   previousPackage=" + previousPackage)
                os.chdir(previousPackage)
                if os.path.exists("debian"):
                    shutil.rmtree("debian")
                cmd = [
                    "tar",
                    "-xvf",
                    os.path.join(
                        toolPath, "skrooge-release-debian-kf" + v["v"] + ".tar.gz"
                    ),
                ]
                self.logfile.write("### " + " ".join(cmd) + "\n")
                rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
                if rc == 0:
                    cmd = ["uupdate", "-u", "skrooge-" + args.version + ".tar.xz"]
                    self.logfile.write("### " + " ".join(cmd) + "\n")
                    rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
                if rc == 0:
                    # Read the changelog
                    os.chdir("../skrooge-" + args.version)
                    if os.path.exists("debian"):
                        shutil.rmtree("debian")
                    cmd = [
                        "tar",
                        "-xvf",
                        os.path.join(
                            toolPath, "skrooge-release-debian-kf" + v["v"] + ".tar.gz"
                        ),
                    ]
                    self.logfile.write("### " + " ".join(cmd) + "\n")
                    rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
                    with open(
                        os.path.join(
                            self.workdingDir, "skrooge-" + args.version + "/CHANGELOG"
                        ),
                        "r",
                    ) as fileChangelog:
                        fileChangelog.readline()  # To pass the first line
                        changelogContent = ""
                        done = False
                        for line in fileChangelog:
                            if line.strip() == "" and done == False:
                                changelogContent += " -- Stéphane MANKOWSKI <stephane@mankowski.fr>  {}\n".format(
                                    datetime.datetime.now().strftime(
                                        "%a, %d %b %Y %H:%M:%S +0100"
                                    )
                                )
                                done = True
                            if not line.startswith(" -- "):
                                changelogContent += line

                        #
                        for ppa in self.ppatotreat:
                            for i in range(len(v["ubuntuVersions"])):
                                print(
                                    "#   {}/{}: {} - {}-kf{}".format(
                                        i + 1,
                                        len(v["ubuntuVersions"]),
                                        v["ubuntuVersions"][i],
                                        ppa,
                                        v["v"],
                                    )
                                )
                                f = open(
                                    os.path.join(
                                        self.workdingDir,
                                        "skrooge-" + args.version + "/debian/control",
                                    ),
                                    "r",
                                )
                                cf = f.read()
                                f.close()

                                # print('#   make changes for '+v["ubuntuVersions"][i])
                                # f = open(os.path.join(self.workdingDir, 'skrooge-'+args.version+'/debian/control'), 'w')
                                # f.write(cf.replace('libqjson-dev, \n', '').replace('libqjson0, ', ''))
                                # f.close()

                                with open(
                                    os.path.join(
                                        self.workdingDir,
                                        "skrooge-" + args.version + "/debian/changelog",
                                    ),
                                    "w",
                                ) as fileDebianChangelog:
                                    fileDebianChangelog.write(
                                        "skrooge ("
                                        + args.version
                                        + "-0ubuntu"
                                        + v["v"]
                                        + "~"
                                        + ppa
                                        + str(i + 1)
                                        + ") "
                                        + v["ubuntuVersions"][i]
                                        + "; urgency=medium\n"
                                    )
                                    fileDebianChangelog.write(changelogContent + "\n")

                                    cmd = ["debuild", "-S", "-sa"]
                                    self.logfile.write("### " + " ".join(cmd) + "\n")
                                    rc = subprocess.call(
                                        cmd, stdout=self.logfile, stderr=self.logfile
                                    )
                                    if rc != 0:
                                        break
                            if rc != 0:
                                break

                        os.chdir(self.workdingDir)
                        print("#   Cleaning")
                        shutil.rmtree("skrooge-" + args.version + ".orig")
                        print("#     1- skrooge-" + args.version + ".orig")
                        shutil.rmtree("skrooge-" + args.version)
                        print("#     2- skrooge-" + args.version)
                        if os.path.exists(previousPackage):
                            shutil.rmtree(previousPackage)
                            print("#     3- " + previousPackage)
                        t = previousPackage.replace(
                            os.path.basename(previousPackage),
                            os.path.basename(previousPackage).replace("-", "_"),
                        )
                        if os.path.exists(t + ".orig.tar.xz"):
                            os.remove(t + ".orig.tar.xz")
                            print("#     4- " + t + ".orig.tar.xz")
        print("#   " + ("DONE" if rc == 0 else "FAILED"))
        return rc

    def publishLaunchpad(self, args):
        print("# Publish on launchpad")
        rc = 0
        if not args.publish:
            print("#   Publication ignored")
        os.chdir(self.workdingDir)
        for v in self.versionstotreat:
            for ppa in self.ppatotreat:
                for i in range(len(v["ubuntuVersions"])):
                    print(
                        "#   {}/{}: {} {}".format(
                            i + 1, len(v["ubuntuVersions"]), v["ubuntuVersions"][i], ppa
                        )
                    )
                    cmd = [
                        "dput",
                        "-f",
                        "ppa:s-mankowski/" + ppa + "-kf" + v["v"],
                        "skrooge_"
                        + args.version
                        + "-0ubuntu"
                        + v["v"]
                        + "~"
                        + ppa
                        + str(i + 1)
                        + "_source.changes",
                    ]
                    if not args.publish:
                        print("#     " + " ".join(cmd) + "\n")
                    else:
                        self.logfile.write("### " + " ".join(cmd) + "\n")
                        rc = subprocess.call(
                            cmd, stdout=self.logfile, stderr=self.logfile
                        )
                        if rc != 0:
                            break
        print("#   " + ("DONE" if rc == 0 else "FAILED"))
        return rc

    def publishKDE(self, args):
        print("# Publish on kde")
        os.chdir(self.workdingDir)
        rc = 0
        if not args.publish:
            print("#   Publication ignored")
        os.chdir(self.workdingDir)
        filename = (
            "skrooge-" + args.version + "-x86_64.AppImage"
            if args.appimage
            else (
                "skrooge_" + args.version + "_amd64.snap"
                if args.snap
                else "skrooge-" + args.version + ".tar.xz"
            )
        )
        self.generatePublicationMessage(args, filename)

        cmd = ["kdecp5", filename, "ftp://upload.kde.org/incoming"]
        if not args.publish:
            print("#     " + " ".join(cmd) + "\n")
        else:
            self.logfile.write("### " + " ".join(cmd) + "\n")
            rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
            if rc != 0:
                print("#   " + " ".join(cmd) + " FAILED. Check if already existing")

            cmd = ["kdecp5", filename + ".sig", "ftp://upload.kde.org/incoming"]
            self.logfile.write("### " + " ".join(cmd) + "\n")
            rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
            if rc != 0:
                print("#   " + " ".join(cmd) + " FAILED. Check if already existing")
        print("#   " + ("DONE" if rc == 0 else "FAILED"))
        return rc

    def buildAppImage(self, args):
        print("# Generate App Image")
        os.chdir(self.workdingDir)
        targteappimage = os.path.join(
            self.workdingDir, "skrooge-" + args.version + "-x86_64.AppImage"
        )
        if os.path.exists(targteappimage):
            os.remove(targteappimage)

        cmd = [
            "wget",
            "-c",
            "-nv",
            "https://raw.githubusercontent.com/probonopd/AppImages/master/pkg2appimage",
        ]
        self.logfile.write("### " + " ".join(cmd) + "\n")
        rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
        if rc == 0:
            for line in fileinput.input(["pkg2appimage"], inplace=True):
                if not "libcurl-slim" in line:
                    print(line, end="")

            os.chmod("./pkg2appimage", 0o775)

            if args.fromlocal:
                print("#   Get appimage.yml from local path")
                cmd = ["cp", os.path.join(localPath, "appimage.yml"), "."]
            else:
                cmd = [
                    "wget",
                    "-c",
                    "-nv",
                    "https://invent.kde.org/office/skrooge/-/raw/master/appimage.yml",
                ]
            self.logfile.write("### " + " ".join(cmd) + "\n")
            rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
            if rc == 0:
                if not args.stable:
                    for line in fileinput.input(["appimage.yml"], inplace=True):
                        print(line.replace("ppa-kf5", "beta-kf5"), end="")

                cmd = ["./pkg2appimage", "appimage.yml"]
                self.logfile.write("### " + " ".join(cmd) + "\n")
                rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
                if rc == 0:
                    appimage_file = [
                        os.path.join(self.workdingDir, "out/" + o)
                        for o in os.listdir("./out")
                        if o.endswith(".AppImage")
                    ][0]

                    shutil.move(appimage_file, targteappimage)
        print("#   " + ("DONE" if rc == 0 else "FAILED"))
        return rc

    def buildSnap(self, args):
        print("# Generate Snap")
        os.chdir(self.workdingDir)
        targteappimage = os.path.join(
            self.workdingDir, "skrooge_" + args.version + "_amd64.snap"
        )
        if os.path.exists(targteappimage):
            os.remove(targteappimage)

        if args.fromlocal:
            print("#   Get snapcraft.yaml from local path")
            cmd = ["cp", os.path.join(localPath, "snapcraft.yaml"), "."]
        else:
            cmd = [
                "wget",
                "-c",
                "-nv",
                "https://invent.kde.org/office/skrooge/-/raw/master/snapcraft.yaml",
            ]
        self.logfile.write("### " + " ".join(cmd) + "\n")
        rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
        if rc == 0:
            for line in fileinput.input(["snapcraft.yaml"], inplace=True):
                print(
                    line.replace(
                        'version: "X.X.X"', 'version: "' + args.version + '"'
                    ).replace(
                        "source: XXX",
                        (
                            "source: " + localPath
                            if args.fromlocal
                            else "source: https://invent.kde.org/office/skrooge.git"
                        ),
                    ),
                    end="",
                )

            cmd = ["snapcraft"]
            self.logfile.write("### " + " ".join(cmd) + "\n")
            rc = subprocess.call(cmd, stdout=self.logfile, stderr=self.logfile)
        print("#   " + ("DONE" if rc == 0 else "FAILED"))

        # TO PUBLISH https://docs.snapcraft.io/build-snaps/c
        #      snapcraft push --release=edge skrooge_*.snap
        return rc


def main():
    parser = argparse.ArgumentParser(
        prog="skrooge-release", description="skrooge release maker"
    )

    # Global arguments
    parser.add_argument("--version", required=True, help="The release version")
    parser.add_argument(
        "--fromtar",
        required=False,
        help="To copy an existing tar file instead of building it from git",
    )
    parser.add_argument("--pwd", required=False, help="The password")
    parser.add_argument(
        "--stable",
        action="store_true",
        help="To define this version as a master version",
    )
    parser.add_argument(
        "--publish", action="store_true", help="To publish on launchpad and KDE"
    )
    parser.add_argument(
        "--appimage", action="store_true", help="To generate the appimage only"
    )
    parser.add_argument("--snap", action="store_true", help="To generate the snap only")
    parser.add_argument(
        "--fromlocal",
        action="store_true",
        help="To generate The snap and the appimage from local path ("
        + localPath
        + ")",
    )
    args = parser.parse_args()

    print("#####################")
    print("# Launching release #")
    print("#####################")
    print("# Version         :" + args.version)
    print("# Stable          :" + ("Y" if args.stable else "N"))
    print("# Publish         :" + ("Y" if args.publish else "N"))
    print("# Appimage        :" + ("Y" if args.appimage else "N"))
    print("# Snap            :" + ("Y" if args.snap else "N"))
    if args.fromlocal:
        print("# From local path :" + localPath)
    # Launch the release
    r = Releasor(args)
    rc = 0
    rc = r.prepareWorkingDirectory(args)
    if rc == 0 and not (args.appimage or args.snap):
        rc = r.makeTarFile(args)
    if rc == 0 and not (args.appimage or args.snap):
        rc = r.updateTarFile(args)
    if rc == 0 and not (args.appimage or args.snap):
        rc = r.buildDebian(args)
    if rc == 0 and args.appimage:
        rc = r.buildAppImage(args)
    if rc == 0 and args.snap:
        rc = r.buildSnap(args)
    if rc == 0 and not (args.appimage or args.snap):
        rc = r.publishLaunchpad(args)
    if rc == 0 and not (args.snap):
        rc = r.publishKDE(args)
    print("#####################")
    print("# End of release    #" if rc == 0 else "# FAILURE         #")
    print("#####################")
    return rc


if __name__ == "__main__":
    sys.exit(main())



#***************************************************************************
#* SPDX-FileCopyrightText: 2024 S. MANKOWSKI stephane@mankowski.fr
#* SPDX-FileCopyrightText: 2024 G. DE BURE support@mankowski.fr
#* SPDX-License-Identifier: GPL-3.0-or-later
#***************************************************************************
# works on all platforms
import os

# get the directory containing your running .sikuli
myPath = os.path.dirname(getBundlePath())
if not myPath in sys.path: sys.path.append(myPath)
import shared
try:
    setAutoWaitTimeout(10)
    
    shared.initAllPlugins()
    
    shared.openAllPages()
    
    shared.openSettings()
    
    click("File-1.png")
    click("PrintPrevie-1.png")
    wait(1)
    type(Key.F4, KEY_ALT)
    
    click("File-1.png")
    click("Print-1.png")
    click("Cancel.png")
    
    shared.close()
    pass
except FindFailed:
    shared.generateErrorCapture("print")
    raise



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#***************************************************************************
#* SPDX-FileCopyrightText: 2024 S. MANKOWSKI stephane@mankowski.fr
#* SPDX-FileCopyrightText: 2024 G. DE BURE support@mankowski.fr
#* SPDX-License-Identifier: GPL-3.0-or-later
#***************************************************************************

import csv
import requests
import urllib3
import ssl

class CustomHttpAdapter (requests.adapters.HTTPAdapter):
    # "Transport adapter" that allows us to use custom ssl_context.

    def __init__(self, ssl_context=None, **kwargs):
        self.ssl_context = ssl_context
        super().__init__(**kwargs)

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = urllib3.poolmanager.PoolManager(
            num_pools=connections, maxsize=maxsize,
            block=block, ssl_context=self.ssl_context)


def get_legacy_session():
    ctx = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
    ctx.options |= 0x4  # OP_LEGACY_SERVER_CONNECT
    session = requests.session()
    session.mount('https://', CustomHttpAdapter(ctx))
    return session

url = 'https://stats.oecd.org/sdmx-json/data/DP_LIVE/.CPI.TOT.AGRWTH.A/OECD?contentType=csv&detail=code&separator=comma&startPeriod=1980'
resp = get_legacy_session().get(url)
reader = csv.reader(resp.text.split('\n'), delimiter=',')
data = {}
next(reader, None)  # skip the headers
for row in reader:
    if len(row)>0:
        if row[0] not in data:
            data[row[0]] = []
        data[row[0]].append((row[5], row[6], 1))
for attr, value in data.items():
    value.sort(key=lambda x: x[0], reverse=True)
    computed = []
    cumul = 1
    for i, x in enumerate(value):
        computed.append((x[0], cumul))
        cumul *= (1+float(x[1])/100)

    with open('inflation_'+attr.lower()+'.csv', 'w') as fw:
        writer = csv.writer(fw, lineterminator="\n")
        writer.writerow(("Date","Price"))
        for row in computed:
            writer.writerow(row)
    print(attr.lower()+',', end = '')
print()



#!/usr/bin/env python3

#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#***************************************************************************
#* SPDX-FileCopyrightText: 2024 S. MANKOWSKI stephane@mankowski.fr
#* SPDX-FileCopyrightText: 2024 G. DE BURE support@mankowski.fr
#* SPDX-License-Identifier: GPL-3.0-or-later
#***************************************************************************
import csv
import enum
import locale
import re
import sys
from collections.abc import Generator, Iterable
from contextlib import contextmanager
from dataclasses import dataclass
from datetime import date, datetime
from typing import TextIO

import bs4
import requests
from requests.models import Response

BASE_URL = "https://bourse.boursobank.com/_formulaire-periode/"
LOOKUP_URL = "https://www.boursorama.com/recherche/"
SYMBOL_RE = re.compile(r"/cours/([a-zA-Z0-9]+)")
NAVIGATION_RE = re.compile(r"Page \d+")


class Interval(enum.Enum):
    daily = "1d"
    monthly = "1mo"
    weekly = "1wk"


INTERVAL_TO_PERIOD = {Interval.daily: 1, Interval.monthly: 30, Interval.weekly: 7}

# Always ask for 3Y, worst case we'll have less
DURATION = "3Y"
EXPECTED_DATE_FORMAT = "%d/%m/%Y"


@dataclass
class TickerValue:
    date: date
    price: float


def ticker_to_url(ticker: str) -> str:
    response = requests.get(
        LOOKUP_URL, params=dict(query=ticker), allow_redirects=False
    )
    if not (response.is_permanent_redirect or response.is_redirect):
        raise ValueError(
            "Ticker search did not return a valid redirect: check the symbol"
        )
    redirect = response.headers.get("Location") or ""
    match = SYMBOL_RE.match(redirect)
    if match is None:
        raise ValueError(
            "Ticker search did not return a valid redirect: check the symbol"
        )
    actual_code = match.groups()[0]
    return actual_code


def request_one_page(
    interval: Interval, symbol: str, start_date: date, page: int | None = None
) -> Response:
    params: dict[str, str | int] = {
        "symbol": symbol,
        "historic_search[start_date]": start_date.strftime(EXPECTED_DATE_FORMAT),
        "historic_search[duration]": DURATION,
        "historic_search[period]": INTERVAL_TO_PERIOD[interval],
    }
    url = BASE_URL
    if page is not None:
        url += f"page-{page}"
    response = requests.get(url, params=params)
    return response


def parse_one_page(
    soup: bs4.BeautifulSoup, from_date: date
) -> Generator[TickerValue, None, date | None]:
    base_div = soup.find("div", {"data-refreshable-id": "historical-period"})
    if base_div is None:
        raise ValueError("Page does not have the expected format")
    rows = base_div.findAll("tr", {"class": "c-table__row"})  # type: ignore[union-attr]
    date = None
    for row in rows:
        cells = row.findAll("td")
        date = datetime.strptime(cells[0].text.strip(), EXPECTED_DATE_FORMAT).date()
        price = cells[1].text.strip().replace(' ', '').replace(',', '.')
        if date < from_date:
            break
        yield TickerValue(date, price)
    return date


def download_history(
    interval: Interval, symbol: str, start_date: date
) -> Generator[TickerValue]:
    response = request_one_page(interval, symbol, start_date)
    soup = bs4.BeautifulSoup(response.text, features="lxml")
    earliest_date = yield from parse_one_page(soup, start_date)
    # Now lookup if we have any more pages
    for page_link in soup.find("div", {"class": "c-pagination"}).findAll("a"):  # type: ignore[union-attr]
        if earliest_date is None or earliest_date < start_date:
            break
        if NAVIGATION_RE.match(page_link["aria-label"]):
            page_num = int(page_link.text)
            if page_num != 1:
                response = request_one_page(interval, symbol, start_date, page=page_num)
                soup = bs4.BeautifulSoup(response.text, features="lxml")
                earliest_date = yield from parse_one_page(soup, start_date)


def dump_to_csv(symbol: str, ticker_values: Iterable[TickerValue], output: TextIO):
    writer = csv.writer(output)
    writer.writerow(("date", "price"))
    for value in ticker_values:
        writer.writerow((value.date, value.price))


if __name__ == "__main__":
    symbol = sys.argv[1]
    from_date = date.fromisoformat(sys.argv[2])
    interval = Interval(sys.argv[3])

    actual_symbol = ticker_to_url(symbol)
    results = download_history(interval, actual_symbol, from_date)
    dump_to_csv(symbol, results, sys.stdout)
